@@ -1,23 +1,8 @@
+// SPDX-FileCopyrightText: 2000-2002 Michael R. Elkins <me@mutt.org>
+// SPDX-FileCopyrightText: 2002-2022 Oswald Buddenhagen <ossi@users.sf.net>
+// SPDX-License-Identifier: GPL-2.0-or-later WITH LicenseRef-isync-GPL-exception
 /*
  * mbsync - mailbox synchronizer
- * Copyright (C) 2000-2002 Michael R. Elkins <me@mutt.org>
- * Copyright (C) 2002-2006,2010-2012 Oswald Buddenhagen <ossi@users.sf.net>
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * As a special exception, mbsync may be linked with the OpenSSL library,
- * despite that library's more restrictive license.
  */
 
 #ifndef COMMON_H
@@ -26,8 +11,14 @@
 #include <autodefs.h>
 
 #include <sys/types.h>
+#include <assert.h>
+#include <limits.h>
 #include <stdarg.h>
+#include <stddef.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <string.h>
 #include <time.h>
 
 typedef unsigned char uchar;
@@ -50,14 +41,67 @@ typedef unsigned long ulong;
 #define shifted_bit(in, from, to) \
 	((int)(((uint)(in) / (from > to ? from / to : 1) * (to > from ? to / from : 1)) & to))
 
+#define BIT_ENUM_VAL(name) \
+	name, \
+	name##_dummy = 2 * name - 1,
+
+#define DEFINE_BIT_ENUM(list) \
+	enum list { \
+		list##_dummy, \
+		list(BIT_ENUM_VAL) \
+	};
+
+#define PFX_BIT_ENUM_VAL(pfx, name) \
+	pfx##_##name, \
+	pfx##_##name##_dummy = 2 * pfx##_##name - 1,
+
+#define PFX_BIT_ENUM_NUM(pfx, name) \
+	pfx##_##name##_bit,
+
+#define DEFINE_PFX_BIT_ENUM(list) \
+	enum list { \
+		list##_dummy, \
+		list(PFX_BIT_ENUM_VAL) \
+	}; \
+	enum { \
+		list(PFX_BIT_ENUM_NUM) \
+		list##_num_bits \
+	};
+
+#define BIT_ENUM_STR(pfx, name) \
+	stringify(name) "\0"
+
+#define BIT_ENUM_STR_OFF(pfx, name) \
+	name##_str_off, \
+	name##_str_off_dummy = name##_str_off + sizeof(stringify(name)) - 1,
+
+#define GET_BIT_ENUM_STR_OFF(pfx, name) \
+	name##_str_off,
+
+#define PFX_BIT_ENUM_STR(pfx, name) \
+	stringify(pfx) "_" stringify(name) "\0"
+
+#define PFX_BIT_ENUM_STR_OFF(pfx, name) \
+	pfx##_##name##_str_off, \
+	pfx##_##name##_str_end = pfx##_##name##_str_off + sizeof(stringify(pfx) "_" stringify(name)) - 1,
+
+#define GET_PFX_BIT_ENUM_STR_OFF(pfx, name) \
+	pfx##_##name##_str_off,
+
+#define static_assert_bits(list, type, field) \
+	static_assert( list##_num_bits <= sizeof(((type){ 0 }).field) * 8, \
+	               stringify(type) "::" stringify(field) " is too small" )
+
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
 # define ATTR_UNUSED __attribute__((unused))
 # define ATTR_NORETURN __attribute__((noreturn))
 # define ATTR_PRINTFLIKE(fmt,var) __attribute__((format(printf,fmt,var)))
+# define ATTR_OPTIMIZE __attribute__((optimize("2")))
 #else
 # define ATTR_UNUSED
 # define ATTR_NORETURN
 # define ATTR_PRINTFLIKE(fmt,var)
+# define ATTR_OPTIMIZE
 #endif
 
 #if defined(__clang__)
@@ -92,44 +136,74 @@ typedef unsigned long ulong;
 
 /* main.c */
 
-#define DEBUG_CRASH     0x01
-#define DEBUG_MAILDIR   0x02
-#define DEBUG_NET       0x04
-#define DEBUG_NET_ALL   0x08
-#define DEBUG_SYNC      0x10
-#define DEBUG_MAIN      0x20
-#define DEBUG_DRV       0x40
-#define DEBUG_DRV_ALL   0x80
-#define DEBUG_ALL       (0xFF & ~(DEBUG_NET_ALL | DEBUG_DRV_ALL))
-#define QUIET           0x100
-#define VERYQUIET       0x200
-#define PROGRESS        0x400
-#define VERBOSE         0x800
-#define KEEPJOURNAL     0x1000
-#define ZERODELAY       0x2000
-#define FORCEASYNC      0x4000
+enum {
+	VERYQUIET,
+	QUIET,
+	TERSE,
+	VERBOSE,
+};
 
+#define options_enum(fn) \
+	fn(DEBUG_MAILDIR) \
+	fn(DEBUG_NET) \
+	fn(DEBUG_NET_ALL) \
+	fn(DEBUG_SYNC) \
+	fn(DEBUG_MAIN) \
+	fn(DEBUG_DRV) \
+	fn(DEBUG_DRV_ALL) \
+	\
+	fn(DEBUG_CRASH) \
+	\
+	fn(PROGRESS) \
+	\
+	fn(DRYRUN) \
+	\
+	fn(EXT_EXIT) \
+	\
+	fn(ZERODELAY) \
+	fn(KEEPJOURNAL) \
+	fn(FORCEJOURNAL) \
+	fn(FORCEASYNC_F) \
+	fn(FORCEASYNC_N) \
+	fn(FAKEEXPUNGE) \
+	fn(FAKEDUMBSTORE)
+DEFINE_BIT_ENUM(options_enum)
+#define FORCEASYNC(b) (FORCEASYNC_F << (b))
+
+#define DEBUG_ANY (DEBUG_MAILDIR | DEBUG_NET | DEBUG_SYNC | DEBUG_MAIN | DEBUG_DRV)
+#define DEBUG_ALL (DEBUG_ANY | DEBUG_CRASH)
+
+// Global options
+extern int Verbosity;
 extern int DFlags;
-extern int JLimit;
+extern int JLimit, JCount;
 extern int UseFSync;
-extern char FieldDelimiter;
 
+// Global constants (inited by main())
 extern int Pid;
 extern char Hostname[256];
 extern const char *Home;
 
-extern uint BufferLimit;
-
-extern int new_total[2], new_done[2];
-extern int flags_total[2], flags_done[2];
-extern int trash_total[2], trash_done[2];
-
+void countStep( void );
 void stats( void );
 
 /* util.c */
 
-void ATTR_PRINTFLIKE(2, 0) vdebug( int, const char *, va_list va );
-void ATTR_PRINTFLIKE(2, 0) vdebugn( int, const char *, va_list va );
+#ifdef DEBUG_FLAG
+#  define debug(...) \
+	do { \
+		if (DFlags & DEBUG_FLAG) \
+			print( __VA_ARGS__ ); \
+	} while (0)
+#  define debugn(...) \
+	do { \
+		if (DFlags & DEBUG_FLAG) \
+			printn( __VA_ARGS__ ); \
+	} while (0)
+#endif
+
+void ATTR_PRINTFLIKE(1, 2) print( const char *, ... );
+void ATTR_PRINTFLIKE(1, 2) printn( const char *, ... );
 void ATTR_PRINTFLIKE(1, 2) info( const char *, ... );
 void ATTR_PRINTFLIKE(1, 2) infon( const char *, ... );
 void ATTR_PRINTFLIKE(1, 2) progress( const char *, ... );
@@ -140,6 +214,17 @@ void ATTR_PRINTFLIKE(1, 0) vsys_error( const char *, va_list va );
 void ATTR_PRINTFLIKE(1, 2) sys_error( const char *, ... );
 void flushn( void );
 
+char *xvasprintf( const char *fmt, va_list ap );
+void xprintf( const char *fmt, ... );
+
+#if !defined(_POSIX_SYNCHRONIZED_IO) || _POSIX_SYNCHRONIZED_IO <= 0
+# define fdatasync fsync
+#endif
+
+void ATTR_PRINTFLIKE(2, 0) vFprintf( FILE *f, const char *msg, va_list va );
+void ATTR_PRINTFLIKE(2, 3) Fprintf( FILE *f, const char *msg, ... );
+void Fclose( FILE *f, int safe );
+
 typedef struct string_list {
 	struct string_list *next;
 	char string[1];
@@ -156,16 +241,54 @@ void *memrchr( const void *s, int c, size_t n );
 size_t strnlen( const char *str, size_t maxlen );
 #endif
 
+void to_upper( char *str, uint len );
+
 int starts_with( const char *str, int strl, const char *cmp, uint cmpl );
 int starts_with_upper( const char *str, int strl, const char *cmp, uint cmpl );
 int equals( const char *str, int strl, const char *cmp, uint cmpl );
+int equals_upper( const char *str, int strl, const char *cmp, uint cmpl );
 
 #ifndef HAVE_TIMEGM
 time_t timegm( struct tm *tm );
 #endif
 
+void fmt_bits( uint bits, uint num_bits, const char *bit_str, const int *bit_off, char *buf );
+
+#define BIT_FORMATTER_RET(name, list, fn) \
+	enum { list(fn##_OFF) name##_str_len }; \
+	struct name##_str { char str[name##_str_len]; };
+
+#define BIT_FORMATTER_PROTO(name, storage) \
+	storage struct name##_str ATTR_OPTIMIZE  /* force RVO */ \
+	fmt_##name( uint bits )
+
+#define BIT_FORMATTER_IMPL(name, list, fn, storage) \
+	BIT_FORMATTER_PROTO(name, storage) \
+	{ \
+		static const char strings[] = list(fn); \
+		static const int offsets[] = { list(GET_##fn##_OFF) }; \
+		\
+		struct name##_str buf; \
+		fmt_bits( bits, as(offsets), strings, offsets, buf.str ); \
+		return buf; \
+	}
+
+// Note that this one uses enum labels without prefix ...
+#define BIT_FORMATTER_FUNCTION(name, list) \
+	BIT_FORMATTER_RET(name, list, BIT_ENUM_STR) \
+	BIT_FORMATTER_IMPL(name, list, BIT_ENUM_STR, static)
+
+// ... while these ones use enum labels with prefix - this
+// is not fundamental, but simply because of the use cases.
+#define DECL_BIT_FORMATTER_FUNCTION(name, list) \
+	BIT_FORMATTER_RET(name, list, PFX_BIT_ENUM_STR) \
+	BIT_FORMATTER_PROTO(name, );
+
+#define DEF_BIT_FORMATTER_FUNCTION(name, list) \
+	BIT_FORMATTER_IMPL(name, list, PFX_BIT_ENUM_STR, )
+
 void *nfmalloc( size_t sz );
-void *nfcalloc( size_t sz );
+void *nfzalloc( size_t sz );
 void *nfrealloc( void *mem, size_t sz );
 char *nfstrndup( const char *str, size_t nchars );
 char *nfstrdup( const char *str );
@@ -175,9 +298,9 @@ int ATTR_PRINTFLIKE(3, 4) nfsnprintf( char *buf, int blen, const char *fmt, ...
 void ATTR_NORETURN oob( void );
 void ATTR_NORETURN oom( void );
 
-char *expand_strdup( const char *s );
+int map_name( const char *arg, int argl, char **result, uint reserve, const char *in, const char *out );
 
-int map_name( const char *arg, char **result, uint reserve, const char *in, const char *out );
+int mkdir_p( char *path, int len );
 
 #define DEFINE_ARRAY_TYPE(T) \
 	typedef struct { \
@@ -251,9 +374,11 @@ typedef struct {
 	list_head_t links;
 	void (*cb)( void *aux );
 	void *aux;
-	time_t timeout;
+	int64_t timeout;
 } wakeup_t;
 
+void init_timers( void );
+int64_t get_now( void );
 void init_wakeup( wakeup_t *tmr, void (*cb)( void * ), void *aux );
 void conf_wakeup( wakeup_t *tmr, int timeout );
 void wipe_wakeup( wakeup_t *tmr );
