@@ -9,7 +9,6 @@
 
 #define nz(a, b) ((a) ? (a) : (b))
 
-static int ops_any[2], trash_any[2], expunge_any[2];
 static int chans_total, chans_done;
 static int boxes_total, boxes_done;
 
@@ -84,25 +83,10 @@ summary( void )
 	if (!boxes_done)
 		return;  // Shut up if we errored out early.
 
-	printf( "Processed %d box(es) in %d channel(s)", boxes_done, chans_done );
-	for (int t = 2; --t >= 0; ) {
-		if (ops_any[t])
-			printf( (DFlags & DRYRUN) ?
-			            ",\nwould %s %d new message(s) and %d flag update(s)" :
-			            ",\n%sed %d new message(s) and %d flag update(s)",
-			        str_hl[t], new_done[t], flags_done[t] );
-		if (trash_any[t])
-			printf( (DFlags & DRYRUN) ?
-			            ",\nwould move %d %s message(s) to trash" :
-			            ",\nmoved %d %s message(s) to trash",
-			        trash_done[t], str_fn[t] );
-		if (expunge_any[t])
-			printf( (DFlags & DRYRUN) ?
-			            ",\nwould expunge %d message(s) from %s" :
-			            ",\nexpunged %d message(s) from %s",
-			        expunge_done[t], str_fn[t] );
-	}
-	puts( "." );
+	printf( "Channels: %d    Boxes: %d    Far: +%d *%d #%d -%d    Near: +%d *%d #%d -%d\n",
+	        chans_done, boxes_done,
+	        new_done[F], flags_done[F], trash_done[F], expunge_done[F],
+	        new_done[N], flags_done[N], trash_done[N], expunge_done[N] );
 }
 
 static int
@@ -244,14 +228,6 @@ add_channel( chan_ent_t ***chanapp, channel_conf_t *chan, int ops[] )
 			free( ce );
 			return NULL;
 		}
-		if (chan->ops[t] & OP_MASK_TYPE)
-			ops_any[t] = 1;
-		if (chan->ops[t] & (OP_EXPUNGE | OP_EXPUNGE_SOLO)) {
-			expunge_any[t] = 1;
-			if (chan->stores[t]->trash ||
-			    (chan->stores[t^1]->trash && chan->stores[t^1]->trash_remote_new))
-				trash_any[t] = 1;
-		}
 	}
 
 	**chanapp = ce;
@@ -474,7 +450,7 @@ do_sync_chans( main_vars_t *mvars )
 			labels[F] = labels[N] = "";
 		for (int t = 0; t < 2; t++) {
 			store_t *ctx = mvars->drv[t]->alloc_store( mvars->chan->stores[t], labels[t] );
-			if ((DFlags & DEBUG_DRV) || ((DFlags & FORCEASYNC(t)) && !(dcaps[t] & DRV_ASYNC))) {
+			if ((DFlags & (DEBUG_DRV | DRYRUN)) || ((DFlags & FORCEASYNC(t)) && !(dcaps[t] & DRV_ASYNC))) {
 				mvars->drv[t] = &proxy_driver;
 				ctx = proxy_alloc_store( ctx, labels[t], DFlags & FORCEASYNC(t) );
 			}
@@ -532,7 +508,7 @@ store_connected( int sts, void *aux )
 					int bufl = snprintf( buf, sizeof(buf), "%s%s", nz( mvars->chan->boxes[t], "" ), pat );
 					int flags = 0;
 					// Partial matches like "INB*" or even "*" are not considered,
-					// except implicity when the INBOX lives under Path.
+					// except implicitly when the INBOX lives under Path.
 					if (starts_with( buf, bufl, "INBOX", 5 )) {
 						char c = buf[5];
 						if (!c) {
