@@ -431,7 +431,7 @@ box_confirmed2( sync_vars_t *svars, int t )
 					/* This can legitimately happen if a deletion propagation was interrupted.
 					 * We have no place to record this transaction, so we just assume it.
 					 * Of course this bears the danger of clearing the state if both mailboxes
-					 * temorarily cannot be opened for some weird reason (while the stores can). */
+					 * temporarily cannot be opened for some weird reason (while the stores can). */
 					delete_state( svars );
 				}
 			  done:
@@ -440,12 +440,12 @@ box_confirmed2( sync_vars_t *svars, int t )
 			}
 			if (svars->existing) {
 				if (!(svars->chan->ops[t^1] & OP_REMOVE)) {
-					error( "Error: channel %s: %s box %s cannot be opened.\n",
+					error( "Error: channel %s: %s box %s cannot be opened anymore.\n",
 					       svars->chan->name, str_fn[t], svars->orig_name[t] );
 					goto bail;
 				}
 				if (svars->drv[t^1]->confirm_box_empty( svars->ctx[t^1] ) != DRV_OK) {
-					warn( "Warning: channel %s: %s box %s cannot be opened and %s box %s is not empty.\n",
+					warn( "Warning: channel %s: %s box %s cannot be opened anymore, and %s box %s is not empty.\n",
 					      svars->chan->name, str_fn[t], svars->orig_name[t], str_fn[t^1], svars->orig_name[t^1] );
 					goto done;
 				}
@@ -511,7 +511,7 @@ box_opened2( sync_vars_t *svars, int t )
 	channel_conf_t *chan;
 	sync_rec_t *srec;
 	uint_array_alloc_t mexcs;
-	uint opts[2], fails, minwuid;
+	uint opts[2], minwuid;
 
 	svars->state[t] |= ST_SELECTED;
 	if (!(svars->state[t^1] & ST_SELECTED))
@@ -520,23 +520,13 @@ box_opened2( sync_vars_t *svars, int t )
 	ctx[1] = svars->ctx[1];
 	chan = svars->chan;
 
-	fails = 0;
-	for (t = 0; t < 2; t++)
-		if (svars->uidval[t] != UIDVAL_BAD && svars->uidval[t] != svars->newuidval[t])
-			fails++;
-	// If only one side changed UIDVALIDITY, we will try to re-approve it further down.
-	if (fails == 2) {
-		error( "Error: channel %s: UIDVALIDITY of both far side %s and near side %s changed.\n",
-		       svars->chan->name, svars->orig_name[F], svars->orig_name[N]);
+	if (!lock_state( svars )) {
 	  bail:
 		svars->ret = SYNC_FAIL;
 		sync_bail( svars );
 		return;
 	}
 
-	if (!lock_state( svars ))
-		goto bail;
-
 	int any_dummies[2] = { 0, 0 };
 	int any_purges[2] = { 0, 0 };
 	int any_upgrades[2] = { 0, 0 };
@@ -574,9 +564,11 @@ box_opened2( sync_vars_t *svars, int t )
 	}
 
 	opts[F] = opts[N] = 0;
-	if (fails)
-		opts[F] = opts[N] = OPEN_PAIRED | OPEN_PAIRED_IDS;
 	for (t = 0; t < 2; t++) {
+		if (svars->uidval[t] != UIDVAL_BAD && svars->uidval[t] != svars->newuidval[t]) {
+			opts[F] |= OPEN_PAIRED | OPEN_PAIRED_IDS;
+			opts[N] |= OPEN_PAIRED | OPEN_PAIRED_IDS;
+		}
 		if (any_purges[t]) {
 			debug( "resuming %d %s purge(s)\n", any_purges[t], str_fn[t] );
 			opts[t] |= OPEN_SETFLAGS;
@@ -859,8 +851,15 @@ box_loaded( int sts, message_t *msgs, int total_msgs, int recent_msgs, void *aux
 				if (!srec->msg[t^1])
 					continue;  // Partner disappeared.
 				if (!srec->msg[t^1]->msgid || strcmp( srec->msg[F]->msgid, srec->msg[N]->msgid )) {
-					error( "Error: channel %s, %s box %s: UIDVALIDITY genuinely changed (at UID %u).\n",
-					       svars->chan->name, str_fn[t], svars->orig_name[t], srec->uid[t] );
+					if (svars->uidval[t^1] != svars->newuidval[t^1]) {
+						error( "Error: channel %s, %s box %s (at UID %u):"
+						       " Unable to recover from both-sided UIDVALIDITY change,"
+						       " as it is genuine on at least one side.\n",
+						       svars->chan->name, str_fn[t], svars->orig_name[t], srec->uid[t] );
+					} else {
+						error( "Error: channel %s, %s box %s (at UID %u): UIDVALIDITY genuinely changed.\n",
+						       svars->chan->name, str_fn[t], svars->orig_name[t], srec->uid[t] );
+					}
 				  uvchg:
 					svars->ret |= SYNC_FAIL;
 					cancel_sync( svars );
@@ -1014,7 +1013,7 @@ box_loaded( int sts, message_t *msgs, int total_msgs, int recent_msgs, void *aux
 						sflags = sanitize_flags( sflags, svars, t );
 						if ((t != xt) && (srec->status & (S_EXPIRE | S_EXPIRED))) {
 							/* Don't propagate deletion resulting from expiration. */
-							debug( "  near side expiring\n" );
+							debug( "  %s side expiring\n", str_fn[xt] );
 							sflags &= ~F_DELETED;
 						}
 						if (srec->status & S_DUMMY(t^1)) {
@@ -1205,7 +1204,7 @@ box_loaded( int sts, message_t *msgs, int total_msgs, int recent_msgs, void *aux
 			if (!srec->uid[xt^1])
 				continue;
 			if (!(srec->status & S_PENDING)) {
-				// We ignore unpaired far-side messages, as there is obviously nothing
+				// We ignore unpaired keep-side messages, as there is obviously nothing
 				// to expire in the first place.
 				if (!srec->msg[xt])
 					continue;
@@ -1230,6 +1229,8 @@ box_loaded( int sts, message_t *msgs, int total_msgs, int recent_msgs, void *aux
 					// but we may be pulling in the real ones.
 					nflags = (srec->pflags | srec->aflags[xt]) & ~srec->dflags[xt];
 				} else {
+					if (!srec->msg[xt^1])
+						continue;
 					nflags = srec->msg[xt^1]->flags;
 				}
 			}
