@@ -1,38 +1,15 @@
-/*
- * mbsync - mailbox synchronizer
- * Copyright (C) 2014 Oswald Buddenhagen <ossi@users.sf.net>
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- * As a special exception, mbsync may be linked with the OpenSSL library,
- * despite that library's more restrictive license.
- */
+// SPDX-FileCopyrightText: 2014-2022 Oswald Buddenhagen <ossi@users.sf.net>
+// SPDX-License-Identifier: GPL-2.0-or-later
+//
+// isync test suite
+//
 
 #include "common.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <time.h>
-
-/* Just to satisfy the references in util.c */
-int DFlags;
-const char *Home;
-
 typedef struct {
 	int id;
 	int first, other, morph_at, morph_to;
-	time_t start;
+	int64_t start;
 	wakeup_t timer;
 	wakeup_t morph_timer;
 } tst_t;
@@ -41,7 +18,7 @@ static void
 timer_start( tst_t *timer, int to )
 {
 	printf( "starting timer %d, should expire after %d\n", timer->id, to );
-	time( &timer->start );
+	timer->start  = get_now();
 	conf_wakeup( &timer->timer, to );
 }
 
@@ -51,7 +28,7 @@ timed_out( void *aux )
 	tst_t *timer = (tst_t *)aux;
 
 	printf( "timer %d expired after %d, repeat %d\n",
-	        timer->id, (int)(time( 0 ) - timer->start), timer->other );
+	        timer->id, (int)(get_now() - timer->start), timer->other );
 	if (timer->other >= 0) {
 		timer_start( timer, timer->other );
 	} else {
@@ -67,7 +44,7 @@ morph_timed_out( void *aux )
 	tst_t *timer = (tst_t *)aux;
 
 	printf( "morphing timer %d after %d\n",
-	        timer->id, (int)(time( 0 ) - timer->start) );
+	        timer->id, (int)(get_now() - timer->start) );
 	timer_start( timer, timer->morph_to );
 }
 
@@ -78,6 +55,7 @@ main( int argc, char **argv )
 {
 	int i;
 
+	init_timers();
 	for (i = 1; i < argc; i++) {
 		char *val = argv[i];
 		tst_t *timer = nfmalloc( sizeof(*timer) );
